/*************************************************************************************************
 * A wrapper of 7zip to use LZMA from C
 *************************************************************************************************/


extern "C" {
#include "mylzma.h"
}

#include "MyWindows.h"
#include "MyInitGuid.h"
#include "CommandLineParser.h"
#include "StringConvert.h"
#include "StringToInt.h"
#include "FileStreams.h"
#include "StreamUtils.h"
#include "LZMADecoder.h"
#include "LZMAEncoder.h"
#include "LzmaBench.h"
#include "LzmaRam.h"
#include <cstdlib>
#include <cstring>

extern "C" {
#include "LzmaRamDecode.h"
}

using namespace std;


/*************************************************************************************************
 * API
 *************************************************************************************************/


/* Compress a serial object by LZMA. */
extern "C"
char *mylzmaencode(const char *ptr, int size, int *sp){
  char* rbuf;
  size_t rsiz, osiz;
  if(size < 0) size = strlen(ptr);
  rsiz = size / 20 * 21 + (1 << 16) + 1;
  if(!(rbuf = (char*)malloc(rsiz))) return NULL;
  if(LzmaRamEncode((Byte*)ptr, size, (Byte*)rbuf, rsiz, &osiz,
                   1 << 23, SZ_FILTER_AUTO) != 0){
    free(rbuf);
    return NULL;
  }
  rbuf[osiz] = '\0';
  *sp = osiz;
  return rbuf;
}


/* Decompress a serial object compressed by LZMA. */
extern "C"
char *mylzmadecode(const char *ptr, int size, int *sp){
  char* rbuf;
  size_t rsiz, osiz;
  if(LzmaRamGetUncompressedSize((const unsigned char*)ptr, size, &rsiz) != 0)
    return NULL;
  if(!(rbuf = (char *)malloc(rsiz + 1))) return NULL;
  if(LzmaRamDecompress((const unsigned char*)ptr, size, (unsigned char *)rbuf, rsiz, &osiz,
                       malloc, free) != 0){
    free(rbuf);
    return NULL;
  }
  rbuf[osiz] = '\0';
  if(sp) *sp = osiz;
  return rbuf;
}



/* END OF FILE */
